
# This file is part of FatxFast.
#
#    FatxFast is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    FatxFast is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with FatxFast.  If not, see <http://www.gnu.org/licenses/>.

# Versioning based on: 
# http://en.wikipedia.org/wiki/Versioning#Designating_development_stage
__author__ = "dryatu (c) 2013"
__version__ = "1.2.5"


import pygame
from FatxFast.objects import entity
from FatxFast.physics import physics
import time


class MoveableEntity(entity.GameEntity):
    """Base class for objects which are moveable on the map. In addition
    to normal GameEntity it has physics
    """
    def __init__(self, worldmap, **kwargs):
        super(MoveableEntity, self).__init__(**kwargs)
        self.worldmap = worldmap 
        self.physics = physics.Physics(self, self.worldmap, 
            collides=kwargs.get("collides", True))

    def move(self, x=0, y=0):
        pos = self.get_position()
        self.set_position([pos[0]+x, pos[1]+y])
    
    def get_speed(self):
        return self.physics.speedvector

    def set_speed(self, x=None, y=None):
        if x is not None:
            self.physics.speedvector.x = x
        if y is not None:
            self.physics.speedvector.y = y

    def get_accel(self):
        return self.physics.accelvector

    def set_accel(self, x=None, y=None):
        if x is not None:
            self.physics.accelvector.x = x
        if y is not None:
            self.physics.accelvector.y = y


    def register_entity_to_worldmap(self, worldmap):
        """Registers a tile based map to the entity. Default would be
        a tilemap"""
        self.worldmap = worldmap

    def register_collision_event(self, entity_id, collisionevent):
        self.physics.collision_map[entity_id] = collisionevent

    def update(self, dt):
        self.physics.update(dt)

