
# This file is part of FatxFast.
#
#    FatxFast is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    FatxFast is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with FatxFast.  If not, see <http://www.gnu.org/licenses/>.

# Versioning based on: 
# http://en.wikipedia.org/wiki/Versioning#Designating_development_stage
__author__ = "dryatu (c) 2013"
__version__ = "1.2.5"

from FatxFast.scenes import (gamescene, menuscenes,
                               popupscenes)
from FatxFast.gui import widgetparser
import pygame

class SceneManager(object):
    
    gamescenes = {
        "menuscene": menuscenes.MenuScene,
        "introscene": menuscenes.IntroScene,
        "savescene": menuscenes.SaveScene,
        "scorescene": menuscenes.ScoreScene,
        "exitscene": menuscenes.ExitScene,
        "configscene": menuscenes.ConfigScene,
        "hotkeyscene": popupscenes.HotkeyScene,
        "askloadscene": popupscenes.AskLoadScene,
        "gamescene": gamescene.GameScene,
        "popupscene": popupscenes.PopupScene,
        "gameendscene": popupscenes.GameEndScene
    }
    def __init__(self, game, **kwargs):
        self.game = game
        self.sceneparser = widgetparser.SceneParser("widgets.xml")
        
        self.scenes = []
        self.push_scene(kwargs.get("start_scene", "introscene"))

    def change_scene(self, scene, **kwargs):
        if isinstance(scene, basestring):
            scene = self.gamescenes[scene](self, **kwargs)
        self.pop_scene()
        self.push_scene(scene)  
        scene.start()
    
    def clear(self):
        del self.game.input.key_listeners[:]
        del self.game.input.mouse_listeners[:]
        del self.game.input.system_listeners[:]
        self.game.input.add_key_listener(self.game)
        self.game.input.add_system_listener(self.game)
        del self.scenes[:]

    def draw(self, surface):
        if self.scenes:
            self.scenes[-1].draw(surface)
    
    def get_scene(self):
        return self.scenes[-1]

    def get_scene_name(self):
        return self.scenes[-1].name

    def pop_scene(self):
        if self.scenes:
            self.scenes[-1].pause()
            self.scenes.pop()
        if self.scenes:
            self.scenes[-1].resume()
    
    def push_scene(self, scene, **kwargs):
        if self.scenes:
            self.scenes[-1].pause()
        if isinstance(scene, basestring):
            scene = self.gamescenes[scene](self, **kwargs)
        scene.start()
        self.scenes.append(scene)
    
    def update(self, dt):
        if self.scenes:
            self.scenes[-1].update(dt)
